export interface OmdbRating {
    Source: string;
    Value: string;
}
export interface OmdbGetResponse {
    Title: string;
    Year: string;
    Rated: string;
    Released: string;
    Runtime: string;
    Genre: string;
    Director: string;
    Writer: string;
    Actors: string;
    Plot: string;
    Language: string;
    Country: string;
    Awards: string;
    Poster: string;
    Metascore: string;
    imdbRating: string;
    imdbVotes: string;
    imdbID: string;
    Website?: string;
    Ratings?: OmdbRating[];
    DVD?: string;
    BoxOffice?: string;
    Production?: string;
    Type: string;
    Response: string;
}
export interface OmdbTvshow extends OmdbGetResponse {
    totalSeasons: string;
}
export interface OmdbEpisode extends OmdbGetResponse {
    Season: string;
    Episode: string;
    seriesID: string;
}
export interface OmdbSeason {
    Title: string;
    Season: string;
    totalSeasons: string;
    Episodes: OmdbEpisode[];
    Response: string;
}
export interface OmdbSearchResult {
    Title: string;
    Year: string;
    imdbID: string;
    Type: string;
    Poster: string;
}
export interface OmdbSearch {
    Search: OmdbSearchResult[];
    totalResults: string;
    Response: string;
}
export interface OmdbError {
    Response: string;
    Error: string;
}
export declare function isError(response: OmdbSearch | OmdbSeason | OmdbTvshow | OmdbGetResponse | OmdbError): response is OmdbError;
export declare function isTvshow(response: OmdbGetResponse): response is OmdbTvshow;
export declare function isMovie(response: OmdbGetResponse): boolean;
export declare function isEpisode(response: OmdbGetResponse): response is OmdbEpisode;
export declare function isGame(response: OmdbGetResponse): boolean;
export declare function assertEpisodeSeasonResponse(response: unknown): response is OmdbSeason[] | OmdbError[];
export declare function assertGetResponse(response: unknown): response is OmdbGetResponse | OmdbError;
export declare function assertSearchResponse(response: unknown): response is OmdbSearch | OmdbError;
